@@ -27,7 +27,9 @@
 #include "cJSON.h"
 #include "loader.h"
 #include "loader_environment.h"
+#if defined(WIN32)
 #include "loader_windows.h"
+#endif
 #include "log.h"
 #include "stack_allocation.h"
 #include "vk_loader_platform.h"
@@ -37,9 +39,8 @@ loader_settings global_loader_settings;
 
 void free_layer_configuration(const struct loader_instance* inst, loader_settings_layer_configuration* layer_configuration) {
     loader_instance_heap_free(inst, layer_configuration->name);
-    layer_configuration->name = NULL;
     loader_instance_heap_free(inst, layer_configuration->path);
-    layer_configuration->path = NULL;
+    memset(layer_configuration, 0, sizeof(loader_settings_layer_configuration));
 }
 
 void free_loader_settings(const struct loader_instance* inst, loader_settings* settings) {
@@ -50,7 +51,6 @@ void free_loader_settings(const struct loader_instance* inst, loader_settings* s
     }
     loader_instance_heap_free(inst, settings->layer_configurations);
     loader_instance_heap_free(inst, settings->settings_file_path);
-    settings->layer_configurations = NULL;
     memset(settings, 0, sizeof(loader_settings));
 }
 
@@ -337,7 +337,7 @@ VkResult get_loader_settings(const struct loader_instance* inst, loader_settings
 
     // Corresponds to the settings object that has no app keys
     int global_settings_index = -1;
-    // Corresponds to the settings object which has a matchign app key
+    // Corresponds to the settings object which has a matching app key
     int index_to_use = -1;
 
     char current_process_path[1024];
@@ -553,7 +553,7 @@ VkResult get_settings_layers(const struct loader_instance* inst, struct loader_l
         }
 
         // The special layer location that indicates where unordered layers should go only should have the
-        // settings_control_value set - everythign else should be NULL
+        // settings_control_value set - everything else should be NULL
         if (layer_config->control == LOADER_SETTINGS_LAYER_UNORDERED_LAYER_LOCATION) {
             struct loader_layer_properties props = {0};
             props.settings_control_value = LOADER_SETTINGS_LAYER_UNORDERED_LAYER_LOCATION;
