@@ -27,6 +27,7 @@
 
 #include "test_environment.h"
 
+#include <fstream>
 #include <mutex>
 
 struct MemoryTrackerSettings {
@@ -434,7 +435,8 @@ TEST(Allocation, CreateInstanceIntentionalAllocFailInvalidManifests) {
 
     for (size_t i = 0; i < invalid_jsons.size(); i++) {
         auto file_name = std::string("invalid_implicit_layer_") + std::to_string(i) + ".json";
-        fs::path new_path = env.get_folder(ManifestLocation::implicit_layer).write_manifest(file_name, invalid_jsons[i]);
+        std::filesystem::path new_path =
+            env.get_folder(ManifestLocation::implicit_layer).write_manifest(file_name, invalid_jsons[i]);
         env.platform_shim->add_manifest(ManifestCategory::implicit_layer, new_path);
     }
 
@@ -525,7 +527,7 @@ TEST(Allocation, CreateInstanceIntentionalAllocFailWithSettingsFilePresent) {
             LoaderSettingsLayerConfiguration{}
                 .set_name(layer_name)
                 .set_control("auto")
-                .set_path(env.get_shimmed_layer_manifest_path(0).str()))));
+                .set_path(env.get_shimmed_layer_manifest_path(0)))));
 
     size_t fail_index = 0;
     VkResult result = VK_ERROR_OUT_OF_HOST_MEMORY;
@@ -561,7 +563,7 @@ TEST(Allocation, CreateSurfaceIntentionalAllocFailWithSettingsFilePresent) {
             LoaderSettingsLayerConfiguration{}
                 .set_name(layer_name)
                 .set_control("auto")
-                .set_path(env.get_shimmed_layer_manifest_path(0).str()))));
+                .set_path(env.get_shimmed_layer_manifest_path(0)))));
 
     size_t fail_index = 0;
     VkResult result = VK_ERROR_OUT_OF_HOST_MEMORY;
@@ -608,7 +610,8 @@ TEST(Allocation, DriverEnvVarIntentionalAllocFail) {
                            "test_layer.json");
     env.get_test_layer().set_do_spurious_allocations_in_create_instance(true).set_do_spurious_allocations_in_create_device(true);
 
-    env.env_var_vk_icd_filenames.add_to_list((fs::path("totally_made_up") / "path_to_fake" / "jason_file.json").str());
+    env.env_var_vk_icd_filenames.add_to_list("totally_made_up/path_to_fake/jason_file.json");
+    env.env_var_vk_icd_filenames.add_to_list("another\\bonkers\\file_path.json");
     size_t fail_index = 0;
     VkResult result = VK_ERROR_OUT_OF_HOST_MEMORY;
     while (result == VK_ERROR_OUT_OF_HOST_MEMORY && fail_index <= 10000) {
@@ -745,8 +748,8 @@ TEST(Allocation, CreateInstanceDeviceIntentionalAllocFail) {
     std::stringstream custom_json_file_contents;
     custom_json_file_contents << custom_json_file.rdbuf();
 
-    fs::path new_path = env.get_folder(ManifestLocation::explicit_layer)
-                            .write_manifest("VK_LAYER_complex_file.json", custom_json_file_contents.str());
+    std::filesystem::path new_path = env.get_folder(ManifestLocation::explicit_layer)
+                                         .write_manifest("VK_LAYER_complex_file.json", custom_json_file_contents.str());
     env.platform_shim->add_manifest(ManifestCategory::explicit_layer, new_path);
 
     size_t fail_index = 0;
